use Test::More tests => 40;
use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Order::API::Order qw ( CreateOrderWithContainer );

sub TestPassingParameters {
    # check erroneously passed parameters
    eval {
        DE_EPAGES::Object::API::Table::Locale::GetLanguageByLocaleID();
    };
    ok( ExistsError() && GetError()->code eq 'PARAMETER_MISSING', 'missing parameter (GetLanguageByLocaleID)' );

    eval {
        DE_EPAGES::Object::API::Table::Locale::GetLanguageByLocaleID(undef);
    };
    ok( ExistsError() && GetError()->code eq 'PARAMETER_MISSING', 'first parameter is undef (GetLanguageByLocaleID)' );

    eval {
        my $Object = {};
        bless ( $Object, 'object' );
        DE_EPAGES::Object::API::Table::Locale::GetLanguageByLocaleID($Object);
    };
    ok( ExistsError() && GetError()->code eq 'STRING_ISOBJECT', 'parameter is no integer (GetLanguageByLocaleID)' );

    return;
}

sub TestLocales {
    # some valid locales
    ## no critic qw(ProhibitMagicNumbers)
    my @aaValidLocales = (
        [ 'de_AT',  1 ],
        [ 'en_NZ',  2 ],
        [ 'fr_BE',  3 ],
        [ 'es_AR',  4 ],
        [ 'ca_ES',  5 ],
        [ 'it_CH',  6 ],
        [ 'sv_FI',  7 ],
        [ 'nl_BE',  8 ],
        [ 'pt_BR',  9 ],
        [ 'da_DK', 10 ],
        [ 'fi_FI', 11 ],
        [ 'no_NO', 12 ],
        [ 'ru_RU', 13 ],
        [ 'ru_UA', 13 ],
        [ 'cs_CZ', 14 ],
    );

    # get language id for these valid locales
    my $LanguageID;
    foreach my $aValidLocale ( @aaValidLocales ) {
        $LanguageID = DE_EPAGES::Object::API::Table::Locale::GetLanguageByLocaleID($aValidLocale->[0]);
        is( $LanguageID, $aValidLocale->[1], 'Get language for locale ' . $aValidLocale->[0] . ' (GetLanguageByLocaleID)');
    }

    # some invalid locales
    my @aInvalidLocales = qw( de_GB en_FR fr_GB en_DE );
    # check, if an error is raised for these invalid locales
    foreach my $InvalidLocale ( @aInvalidLocales ) {
        eval {
            $LanguageID = DE_EPAGES::Object::API::Table::Locale::GetLanguageByLocaleID($InvalidLocale);
        };
        ok( ExistsError() && GetError()->code eq 'NoResults', 'Check invalid locale ' . $InvalidLocale );
    }

    # create a lineitemcontainer in order to check $Container->get('LanguageID')
    my $System = LoadRootObject();
    my $Shop = $System->child( 'TestShop' );
    my $CustomerNo = GetNextCustomerNo( $Shop );
    my $CustomerFolder = GetCustomerFolder( $Shop );
    my $Customer = $Shop->get('CustomerClass')->insertObject({
        Parent => $CustomerFolder,
        Alias  => $CustomerNo,
    });
    my $Order = CreateOrderWithContainer($Customer);
    my $Container = $Order->get('LineItemContainer');

    foreach my $aValidLocale ( @aaValidLocales ) {
        $Container->set({ 'LocaleID' => $aValidLocale->[0] });
        $LanguageID = $Container->get('LanguageID');
        is( $LanguageID, $aValidLocale->[1], 'Get language for locale ' . $aValidLocale->[0] . ' (Container->get)');
    }

    # setting the languageid on the lineitemcontainer results in an error
    eval {
        $Container->set({ 'LanguageID' => 1 });
    };
    ok( ExistsError() && GetError()->code eq 'ReadOnlyAttribute', 'Set language id');

    return;
}

sub Cleanup {
    my $System = LoadRootObject();
    if( $System->existsChild('TestShop') ) {
        $System->child('TestShop')->delete;
    }

    return 1;
}

sub Init {
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild('Shop', { Alias => 'TestShop' } );

    return 1;
}

RunScript( Message => 'long', Sub => sub {
    RunOnStore(
        Store => GetTestSite(),
        Sub   => sub {
            ok( Init(), 'Initialization' );

            TestPassingParameters();
            TestLocales();

            ok( Cleanup(), 'Cleanup' );
        }
    );
});

1;