#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::SearchParamBuilder
# state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::SearchParamBuilder;
use base Test::Class;

use strict;
use Test::More;
use DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );

sub setup : Test(setup) {
    my $self = shift;
    $self->{builder} = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new();
}

sub builder {
    my $self = shift;
    return $self->{builder};
}

sub testBuild : Test(no_plan) {
    my $self = shift;
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    is_deeply($self->builder->params(),  {'attributeFilters'=>[]}, 'attributeFilters is empty');

    my $hParams = $self->builder->lang('de')
        ->query('  ') # testing empty query
        ->currency('EUR')
        ->sortBy('RELEVANCE')
        ->currentPage(1)
        ->addTaxAreaID(5)
        ->params()
    ;

    is_deeply( $hParams, {
        'attributeFilters' => [],
        'lang' => 'de',
        'currency' => 'EUR',
        'sortBy' => 'RELEVANCE',
        'currentPage' => 1,
        'taxAreaId' => 5,
    }, 'built searchparams' );
}

sub testWrongSortBy : Test {
    my $self = shift;
    $self->builder->sortBy('WrongSortKey');
    my $hParams = $self->builder->params();
    is($hParams->{sortBy}, 'RELEVANCE', 'wrong sortBy key default to RELEVANCE');
}

sub testFilterRange : Test {
    my $self = shift;
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    $self->builder->addFilterRange( 'Range_ListPrice', [11, 23.56]);
    is_deeply($self->builder->getFilters(), [
          {
            'attributeValue' => undef,
            'attributeRange' => {
                                  'min' => 11,
                                  'max' => '23.56'
                                },
            'fieldName' => 'Range_ListPrice'
          }
        ],
        'addFilterRange');
}

sub testInvalidRange : Test {
    my $self = shift;
    $self->builder->addFilterRange( 'Range_NullPrice', ['', '']);
    is_deeply($self->builder->getFilters(), [], 'ignore invalid range' );
}

sub testFilterValuesNumberAndString : Test {
    my $self = shift;
    $self->builder->addFilterValues( 'Value_Numbers', ['1', 3.11] );
    is_deeply($self->builder->getFilters(), [
          {
            'attributeValue' => '1',
            'fieldName' => 'Value_Numbers',
            'attributeRange' => undef,
          },
          {
            'attributeValue' => 3.11,
            'fieldName' => 'Value_Numbers',
            'attributeRange' => undef,
          },
        ],
        'testeFilterValuesNumberAndString');
}

sub testFilterRangInvalidOrder : Test(no_plan) {
    my $self = shift;
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    eval { $self->builder->addFilterRange( 'OutOfOrder', [25.0, 20.0] ); };
    ok( !ExistsError, 'addFilterRange' ) or diag( GetError->message );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{min}, 20, 'addFilterRange - reorder min/max' );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{max}, 25, 'addFilterRange - reorder min/max' );
}

sub testHalfOpenRangeWithUndef : Test(no_plan) {
    my $self = shift;
    eval { $self->builder->addFilterRange( 'HalfOpen', [22.0, undef] ); };
    ok( !ExistsError, 'addFilterRange' ) or diag( GetError->message );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{min}, 22,    'addFilterRange - test with undef right (1)' );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{max}, undef, 'addFilterRange - test with undef right (2)' );
}

sub testHalfOpenWithEmptyString : Test(no_plan) {
    my $self = shift;
    eval { $self->builder->addFilterRange( 'HalfOpen', [22.0, ''] ); };
    ok( !ExistsError, 'addFilterRange - half open right' ) or diag( GetError->message );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{min}, 22,    'addFilterRange - test with empty stirng right (1)' );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{max}, undef, 'addFilterRange - test with empty stirng right (2)' );
}

sub testHalfOpenWithoutLowerLimit : Test(no_plan) {
    my $self = shift;
    eval { $self->builder->addFilterRange( 'HalfOpen', ['', 22.0] ); };
    ok( !ExistsError, 'addFilterRange - half open left' ) or diag( GetError->message );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{min}, undef,    'addFilterRange - test with empty stirng left (1)' );
    is( $self->builder->getFilters()->[0]->{attributeRange}->{max}, 22.0, 'addFilterRange - test with empty stirng left (2)' );
}

sub testFacetMode : Test {
    my $self = shift;
    $self->builder()->addFacetMode(MockShop->new());
    is_deeply ($self->builder->params()->{facetMode}, 'ALL_FACETS', 'all facets available');
}

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);


package MockShop; ## no critic (ProhibitMultiplePackages)

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless {}, $class;
    return $self;
}

sub featureMaxValue {
    my $self = shift;
    return 1;
}
