use Test::More tests => 26;
use strict;
use warnings;

use Config;
use English;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw ( SystemCommand );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::File qw ( CreateDirectory DeleteDirectory DeleteFile AbsolutePath );
use DE_EPAGES::ShopTransfer::API::BCP qw ( BCPInIterator );

use Readonly;
Readonly::Scalar my $DEFAULT_OUT   => './out';
Readonly::Scalar my $DE_EPAGES     => $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES';
Readonly::Scalar my $PERL          => $EXECUTABLE_NAME;
Readonly::Scalar my $SHOPALIAS     => 'DemoShop';
Readonly::Scalar my $NEW_SHOPALIAS => 'DemoShopX';
Readonly::Scalar my $SHOP_TXT      => 'shop.txt';
Readonly::Scalar my $STORENAME     => 'Store';

sub TestTransformShop {
    _exportShop();
    ok( 1, 'export shop' );
    _exportSystem();
    ok( 1, 'export system' );

    # (1) perform plain shop transfer without modifying the shop alias
    _transformShop();
    ok( 1, 'transform shop' );
    _validate( $SHOPALIAS );

    # clear output for next test
    my $ImportPath = join('/', $DEFAULT_OUT, $STORENAME, 'import');
    DeleteDirectory( $ImportPath, { 'recursive' => 1 } );


    # (2) perform shop transfer and rename old shop alias
    _transformShopAndRename( $NEW_SHOPALIAS );
    ok( 1, 'transform shop and rename alias' );
    _validate( $NEW_SHOPALIAS );

    # clear output for next test
    DeleteDirectory( $ImportPath, { 'recursive' => 1 } );


    # (3) perform shop transfer and rename old shop alias with creating new guids
    _transformShopAndRenameWithNewGUID( $NEW_SHOPALIAS );
    ok( 1, 'transform shop and rename alias with creating new guids' );
    _validate( $NEW_SHOPALIAS );

    return;
}

sub _validate {
  my ($ShopAlias) = @_;

  # validate import directory having the name of the shop alias
    my $Path = join('/', $DEFAULT_OUT, $STORENAME, 'import', $ShopAlias);
    ok( -d $Path, 'directory with shop alias exists' );

    # validate correct shop alias in object.bcp and site.bcp
    my $Filename = join('/', $Path, 'object.bcp');
    ok( -e $Filename, 'object.bcp exists' );

    my $ContainsShopAlias = 0;
    BCPInIterator( $Filename, sub {
        $ContainsShopAlias = 1 if ($_[3] eq $ShopAlias );
    });
    ok( $ContainsShopAlias, "object.bcp contains the alias $ShopAlias" );

    $ContainsShopAlias = 0;
    $Filename =~ s/object/site/g;
    ok( -e $Filename, 'site.bcp exists' );
    BCPInIterator( $Filename, sub {
        $ContainsShopAlias = 1 if ($_[10] eq $ShopAlias );  ## no critic qw(ProhibitMagicNumbers)
    });
    ok( $ContainsShopAlias, "site.bcp contains the alias $ShopAlias" );

    # extract public.tar in order to perform the following test
    my $SourceTar = AbsolutePath( join('/', $Path, 'public.tar') );
    my $TmpDirectory = join('/', $DEFAULT_OUT, 'temp');
    CreateDirectory( $TmpDirectory );
    my $Tar = $Config{osname} eq 'MSWin32' ? File::Spec->catfile($ENV{'EPAGES'},'bin','tar.exe') : 'tar';
    my $Command = sprintf '%s -xf %s -C %s', $Tar, $SourceTar, $TmpDirectory;
    SystemCommand( $Command );

    # validate correct shop alias within the *.tar archive
    $Path = join('/', $TmpDirectory, $ShopAlias);
    ok( -d $Path, "directory with shop alias '$ShopAlias' exists within tar archive" );

    return;
}

sub _exportShop {
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/export.pl",
        -storename => $STORENAME,
        -shop      => $SHOPALIAS,
    ]);

    return;
}

sub _exportSystem {
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/export.pl",
        -storename => $STORENAME,
        '-system',
    ]);

    return;
}

sub _transformShop {
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/transform.pl",
        -storename => $STORENAME,
        -source    => $STORENAME,
        -shop      => $SHOPALIAS,
    ]);

    return;
}

sub _transformShopAndRename {
    open my $fh, '>', $SHOP_TXT or croak("could not open $SHOP_TXT");
    print $fh "$SHOPALIAS=$NEW_SHOPALIAS";
    close $fh or croak("could not close $SHOP_TXT");

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/transform.pl",
        -storename => $STORENAME,
        -source    => $STORENAME,
        -list      => $SHOP_TXT,
    ]);

    return;
}

sub _transformShopAndRenameWithNewGUID {
    open my $fh, '>', $SHOP_TXT or croak("could not open $SHOP_TXT");
    print $fh "$SHOPALIAS=$NEW_SHOPALIAS";
    close $fh or croak("could not close $SHOP_TXT");

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/transform.pl",
        -storename => $STORENAME,
        -source    => $STORENAME,
        -list      => $SHOP_TXT,
        '-newguid',
    ]);

    return;
}

sub Init {
    Cleanup();

    ok( 1, 'init' );
    return;
}

sub Cleanup {
    DeleteFile( $SHOP_TXT );
    DeleteDirectory( $DEFAULT_OUT, { 'recursive' => 1 } );

    ok( 1, 'cleanup' );
    return;
}

RunScript(
    Sub => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub => sub {
                Init();

                TestTransformShop();

                Cleanup();
            }
        )
    },
    Debug => 1,
);

1;
